/**
 * Copyright (C) 2008 Ovea <dev@testatoo.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.testatoo.cartridge.html4.evaluator.selenium;

import com.thoughtworks.selenium.Selenium;
import com.thoughtworks.selenium.Wait;
import org.testatoo.cartridge.html4.By;
import org.testatoo.cartridge.html4.EvaluatorException;
import org.testatoo.cartridge.html4.HtmlEvaluator;
import org.testatoo.cartridge.html4.component.ListBox;
import org.testatoo.cartridge.html4.element.*;
import org.testatoo.cartridge.html4.element.CheckBox;
import org.testatoo.cartridge.html4.element.Map;
import org.testatoo.cartridge.html4.element.Object;
import org.testatoo.core.AbstractEvaluator;
import org.testatoo.core.ComponentType;
import org.testatoo.core.component.*;
import org.testatoo.core.component.AlertBox;
import org.testatoo.core.component.Button;
import org.testatoo.core.component.DialogBox;
import org.testatoo.core.component.Link;
import org.testatoo.core.component.Radio;
import org.testatoo.core.component.datagrid.*;
import org.testatoo.core.input.Click;
import org.testatoo.core.input.Key;
import org.testatoo.core.nature.*;

import java.util.*;

import static org.testatoo.core.ComponentType.AlertBox;
import static org.testatoo.core.input.KeyModifier.*;

/**
 * This class is the implementation of an evaluator for html4 elements with Selenium as UI Test engine.
 *
 * @author dev@testatoo.org
 */
public final class SeleniumHtmlEvaluator extends AbstractEvaluator<Selenium> implements HtmlEvaluator {

    private final Selenium selenium;
    private Component currentFocusedComponent;

    private java.util.Map<String, String> alertBoxMessage = new HashMap<String, String>();

    /**
     * Class constructor specifying the used selenium engine
     *
     * @param selenium the selenium engine
     */
    public SeleniumHtmlEvaluator(Selenium selenium) {
        this.selenium = selenium;
    }

    /**
     * @see org.testatoo.core.Evaluator
     */
    @Override
    public Selenium implementation() {
        return selenium;
    }


    /**
     * @see org.testatoo.core.Evaluator
     */
    @Override
    public Boolean existComponent(String id) {
        if (id.startsWith(org.testatoo.cartridge.html4.element.AlertBox.ID)) {
            alertBoxMessage.clear();
            return selenium.isAlertPresent();
        }
        // Cannot use jQuery cause only present after page loaded (not the case when existComponent is a page)
        return selenium.isElementPresent("id=" + id);
    }

    /**
     * @see org.testatoo.core.Evaluator
     */
    @Override
    public String text(Button button) {
        String nodeName = selenium.getEval("window.tQuery('#" + button.id() + "').attr('nodeName');");
        if (nodeName.equalsIgnoreCase("input")) {
            return attribute(button.id(), Attribute.value);
        }
        return nodeTextContent(button);
    }

    /**
     * @see org.testatoo.core.Evaluator
     */
    @Override
    public String icon(Button button) {
        String nodeName = selenium.getEval("window.tQuery('#" + button.id() + "').attr('nodeName');");
        if (nodeName.equalsIgnoreCase("input")) {
            return attribute(button.id(), Attribute.src);
        } else {
            // the button tag is used
            return attribute(elementId("jquery:#" + button.id() + " img"), Attribute.src);
        }
    }

    /**
     * @see org.testatoo.core.Evaluator
     */
    @Override
    public Boolean isVisible(Component component) {
        return component instanceof AbstractWindow
                || selenium.isVisible(component.id());
//            || (Boolean.valueOf(selenium.getEval("window.tQuery('#" + component.id() + "').is(':visible') || !(window.tQuery('#" + component.id() + "').is(':hidden')" +
//            " || window.tQuery('#" + component.id() + "').css('visibility') == 'hidden' || window.tQuery('#" + component.id() + "').css('display') == 'none')")));
    }

    /**
     * @see org.testatoo.core.Evaluator
     */
    @Override
    public Boolean isEnabled(Component component) {
        return !Boolean.valueOf(selenium.getEval("window.tQuery('#" + component.id() + "').is(':disabled');"))
                && !Boolean.valueOf(selenium.getEval("window.tQuery('#" + component.id() + "').attr('readonly') == true;"));
    }

    /**
     * @see org.testatoo.core.Evaluator
     */
    @Override
    public void check(Checkable checkable) {
        if (!checkable.isChecked())
            click((Component) checkable, Click.left);
    }

    /**
     * @see org.testatoo.core.Evaluator
     */
    @Override
    public Boolean isChecked(Checkable checkable) {
        return Boolean.valueOf(selenium.getEval("window.tQuery('#" + ((Component) checkable).id() + "').is(':checked');"));
    }

    /**
     * @see org.testatoo.core.Evaluator
     */
    @Override
    public void unCheck(org.testatoo.core.component.CheckBox checkbox) {
        if (checkbox.isChecked())
            click(checkbox, Click.left);
    }

    /**
     * @see org.testatoo.core.Evaluator
     */
    @Override
    public String value(ValueSupport valueSupport) {
        if (valueSupport instanceof AbstractTextField) {
            return selenium.getValue(((Component) valueSupport).id());
        }
        if (valueSupport instanceof Cell) {
            return nodeTextContent(((Component) valueSupport));
        }

        return attribute(((Component) valueSupport).id(), Attribute.value);
    }

    /**
     * @see org.testatoo.core.Evaluator
     */
    @Override
    public String source(org.testatoo.core.component.Image image) {
        return attribute(image.id(), Attribute.src);
    }

    /**
     * @see org.testatoo.core.Evaluator
     */
    @Override
    public String label(LabelSupport labelSupport) {
        try {
            Component label = By.jQuery("label[for=" + ((Component) labelSupport).id() + "]").findComponent(this);
            return nodeTextContent(label);
        } catch (EvaluatorException e) {
            try {
                Component label = By.xpath("//label/*[@id='" + ((Component) labelSupport).id() + "']/parent::*").findComponent(this);
                return nodeTextContent(label);
            } catch (EvaluatorException ex) {
                return "";
            }
        }
    }

    /**
     * @see org.testatoo.core.Evaluator
     */
    @Override
    public Integer maxLength(AbstractTextField textfield) {
        if (attribute(textfield.id(), Attribute.maxlength).equals(""))
            return Integer.MAX_VALUE;
        return Integer.valueOf(attribute(textfield.id(), Attribute.maxlength));
    }

    /**
     * @see org.testatoo.core.Evaluator
     */
    @Override
    public void reset(AbstractTextField textField) {
        selenium.getEval("window.tQuery('#" + textField.id() + "').val('');");
    }

    /**
     * @see org.testatoo.core.Evaluator
     */
    @Override
    public void selectFilePath(String filePath, FileDialog fileDialog) {
        throw new EvaluatorException("Not available for security constraints");
    }

    /**
     * @see org.testatoo.core.Evaluator
     */
    @Override
    public String selectedFilePath(FileDialog fileDialog) {
        throw new EvaluatorException("Not available for security constraints");
    }

    /**
     * @see org.testatoo.core.Evaluator
     */
    @Override
    public void unselect(String value, ListModel listModel) {
        Select select = findEmbeddedSelect(listModel);
        String[] values = parseCSV(selenium.getEval("window.tQuery.map(window.tQuery('#" + select.id() + " :selected'), function(e) { return window.tQuery(e).val(); });"));
        for (Option option : select.options()) {
            List<String> selectedValues = Arrays.asList(values);
            if (option.value().equals(value)) {
                if (selectedValues.contains(option.value())) {
                    selenium.getEval("window.tQuery('#" + option.id() + "').attr('selected', '')");
                    selenium.getEval("window.tQuery('#" + select.id() + "').simulate('change');");
                }
            }
        }
        waitForCondition();
    }

    /**
     * @see org.testatoo.core.Evaluator
     */
    @Override
    public void unselectAll(ListModel listModel) {
        Select select = findEmbeddedSelect(listModel);
        String[] values = parseCSV(selenium.getEval("window.tQuery.map(window.tQuery('#" + select.id() + " :selected'), function(e) { return window.tQuery(e).val(); });"));
        for (Option option : select.options()) {
            List<String> selectedValues = Arrays.asList(values);
            if (selectedValues.contains(option.value())) {
                selenium.getEval("window.tQuery('#" + option.id() + "').attr('selected', '')");
                selenium.getEval("window.tQuery('#" + select.id() + "').simulate('change')");
            }
        }
        waitForCondition();
    }

    /**
     * @see org.testatoo.core.Evaluator
     */
    @Override
    public void select(String value, ListModel listModel) {
        Select select = findEmbeddedSelect(listModel);
        for (Option option : select.options()) {
            if (option.value().equals(value)) {
                selenium.getEval("window.tQuery('#" + option.id() + "').attr('selected', 'selected');");
                // use fix for IE
                selenium.getEval("if (window.tQuery.browser.msie) {window.tQuery('#" + select.id() + "').simulate('click');} " +
                        "else {window.tQuery('#" + select.id() + "').simulate('change');}");
            }
        }
        waitForCondition();
    }

    /**
     * @see org.testatoo.core.Evaluator
     */
    @Override
    public Boolean contains(Container container, Component component) {
        return selenium.isElementPresent("//*[@id='" + ((Component) container).id() + "']//*[@id='" + component.id() + "']");
    }

    /**
     * @see org.testatoo.core.Evaluator
     */
    @Override
    public String label(Option option) {
        return attribute(option.id(), Attribute.label);
    }

    /**
     * @see org.testatoo.core.Evaluator
     */
    @Override
    public String label(OptionGroup optionGroup) {
        return attribute(optionGroup.id(), Attribute.label);
    }

    /**
     * @see org.testatoo.core.Evaluator
     */
    @Override
    public Boolean selected(Option option) {
        String id = option.id();
        return existComponent(id) && (attribute(id, Attribute.selected).toLowerCase().equalsIgnoreCase("selected")
                || attribute(id, Attribute.selected).toLowerCase().equals("true"));
    }

    /**
     * @see org.testatoo.core.Evaluator
     */
    @Override
    public Boolean hasFocus(Component component) {
        return currentFocusedComponent != null && currentFocusedComponent.equals(component);
    }

    /**
     * @see org.testatoo.core.Evaluator
     */
    @Override
    public String text(Link link) {
        return nodeTextContent(link);
    }

    /**
     * @see org.testatoo.core.Evaluator
     */
    @Override
    public String title(TitleSupport titleSupport) {
        if (titleSupport instanceof Page) {
            return selenium.getTitle();
        }

        if (titleSupport instanceof FileDialog) {
            return "";
        }

        if (titleSupport instanceof AlertBox || titleSupport instanceof Prompt || titleSupport instanceof DialogBox) {
            selenium.isAlertPresent();
            return "";
        }

        if (titleSupport instanceof Column) {
            return nodeTextContent((Component) titleSupport);
        }
        return attribute(((Component) titleSupport).id(), Attribute.title);
    }

    /**
     * @see org.testatoo.core.Evaluator
     */
    @Override
    public String message(AlertBox alertbox) {
        String key = alertbox.id().substring(12);
        if (selenium.isAlertPresent()) {
            alertBoxMessage.put(key, selenium.getAlert());
        }
        return (alertBoxMessage.get(key) != null) ? alertBoxMessage.get(key) : "";
    }

    /**
     * @see org.testatoo.core.Evaluator
     */
    @Override
    public List<Column> columns(DataGrid datagrid) {
        String xpath = "//table[@id='" + datagrid.id() + "']/thead/tr[last()]/th";
        int numberOfColumns = selenium.getXpathCount(xpath).intValue();

        List<Column> columns = new ArrayList<Column>();

        for (int rowNum = 1; rowNum <= numberOfColumns; rowNum++) {
            Column column = new Column(this, elementId("xpath:" + xpath + "[" + rowNum + "]"));
            columns.add(column);
        }
        return columns;
    }

    /**
     * @see org.testatoo.core.Evaluator
     */
    @Override
    public List<Row> rows(DataGrid datagrid) {
        List<Row> rows = new ArrayList<Row>();

        String xpath = "//table[@id='" + datagrid.id() + "']/tbody/tr";
        int numberOfRows = selenium.getXpathCount(xpath).intValue();

        for (int rowNum = 1; rowNum <= numberOfRows; rowNum++) {
            Row row = new Row(this, elementId("xpath:" + xpath + "[" + rowNum + "]"));
            rows.add(row);
        }
        return rows;
    }

    @Override
    public List<Cell> cells(CellContainer cellContainer) {
        List<Cell> cells = new ArrayList<Cell>();

        if (cellContainer instanceof Row) {
            String xpath = "//tr[@id='" + ((Component) cellContainer).id() + "']/td";
            int numberOfCells = selenium.getXpathCount(xpath).intValue();

            for (int cellNum = 1; cellNum <= numberOfCells; cellNum++) {
                Cell cell = new Cell(this, elementId("xpath:" + xpath + "[" + cellNum + "]"));
                cells.add(cell);
            }
        }

        if (cellContainer instanceof Column) {
            // Find column number
            String xpath = "//th[@id='" + ((Component) cellContainer).id() + "']/parent::*/th";
            int numberOfColumns = selenium.getXpathCount(xpath).intValue();

            int selectedColumnNum = 0;
            boolean columnNumFind = false;

            for (int colNum = 1; colNum <= numberOfColumns; colNum++) {
                if (elementId("xpath:" + xpath + "[" + colNum + "]").equals(((Component) cellContainer).id())) {

                    selectedColumnNum = colNum;
                    columnNumFind = true;
                }
            }

            if (!columnNumFind) {
                throw new EvaluatorException("Unable to find the Column");
            }

            xpath = "//th[@id='" + ((Component) cellContainer).id() + "']/ancestor::table/tbody/tr";
            int numberOfRows = selenium.getXpathCount(xpath).intValue();

            for (int rowNum = 1; rowNum <= numberOfRows; rowNum++) {
                Cell cell = new Cell(this, elementId("xpath:" + xpath + "[" + rowNum + "]/td[" + selectedColumnNum +"]"));
                cells.add(cell);
            }
        }
        return cells;
    }

    /**
     * @see org.testatoo.core.Evaluator
     */
    @Override
    public ComponentType componentType(String id) {
        if (id.equals(org.testatoo.cartridge.html4.element.AlertBox.ID))
            return AlertBox;
        return ComponentType.valueOf(selenium.getEval("window.tQuery('#" + id + "').componentType()"));
    }

    /**
     * @see org.testatoo.core.Evaluator
     */
    @Override
    public void click(Component component, Click which) {
        try {
            if (which == Click.right) {
                selenium.getEval("window.tQuery('#" + component.id() + "').simulate('rightclick');");
            } else {
                selenium.getEval("window.tQuery('#" + component.id() + "').simulate('click');");
            }
            setFocus(component);
            waitForCondition();
            // If component is link we need to open the expected target
            // But some framework like Wicket use js and set ref to # (in this case we let the framework make the job)
            if (component instanceof Link && !((Link) component).reference().equals("#")) {
                open(((Link) component).reference());
            }
        } catch (Exception e) {
            // Continue .. if the click change page
        }
    }

    /**
     * @see org.testatoo.core.Evaluator
     */
    @Override
    public void doubleClick(Component component) {
        selenium.getEval("window.tQuery('#" + component.id() + "').simulate('dblclick')");
        setFocus(component);
        waitForCondition();
    }

    /**
     * @see org.testatoo.core.Evaluator
     */
    @Override
    public void mouseOver(Component component) {
        selenium.getEval("window.tQuery('#" + component.id() + "').simulate('mouseover')");
        waitForCondition();
    }

    /**
     * @see org.testatoo.core.Evaluator
     */
    @Override
    public void mouseOut(Component component) {
        selenium.getEval("window.tQuery('#" + component.id() + "').simulate('mouseout')");
        waitForCondition();
    }

    /**
     * @see org.testatoo.core.Evaluator
     */
    @Override
    public void dragAndDrop(Component from, Component to) {
        selenium.getEval("window.tQuery('#" + from.id() + "').simulate('dragTo', {'target': window.tQuery('#" + to.id() + "')})");
        waitForCondition();
    }

    /**
     * @see org.testatoo.core.Evaluator
     */
    @Override
    public void focusOn(Component component) {
        click(component, Click.left);
    }

    /**
     * @see org.testatoo.core.Evaluator
     */
    @Override
    public void type(String text) {
        String keyModifier = keyModifier();
        if (currentFocusedComponent != null) {
            for (byte charCode : text.getBytes()) {
                if (Boolean.valueOf(selenium.getEval("window.tQuery.browser.msie"))) {
                    selenium.getEval("window.tQuery('#" + currentFocusedComponent.id() + "')" +
                            ".val(window.tQuery('#" + currentFocusedComponent.id() + "').val() + String.fromCharCode(" + charCode + "));");
                }
                selenium.getEval("window.tQuery('#" + currentFocusedComponent.id() + "').simulate('type', {charCode: " + charCode + keyModifier + "});");
            }
        } else {
            for (char charCode : text.toCharArray()) {
                selenium.getEval("if (window.tQuery.browser.mozilla) {window.tQuery(window.document).simulate('type', {keyCode: " + keyboardLayout.convert(charCode) + keyModifier + "})}" +
                        "else {window.tQuery(window.document).simulate('type', {charCode: " + keyboardLayout.convert(charCode) + keyModifier + "})};");
            }
        }
        waitForCondition();
    }

    /**
     * @see org.testatoo.core.Evaluator
     */
    @Override
    public void press(Key key) {
        typeKey(key.code());
    }

    /**
     * @see org.testatoo.core.Evaluator
     */
    @Override
    public void close(AbstractWindow window) {
        if (window instanceof AlertBox && selenium.isAlertPresent()) {
            selenium.getAlert();
        }
    }

    /**
     * @see org.testatoo.core.Evaluator
     */
    @Override
    public String reference(Link link) {
        return attribute(link.id(), Attribute.href);
    }

    /**
     * @see org.testatoo.cartridge.html4.HtmlEvaluator
     */
    @Override
    public void open(String url) {
        // Selenium issue !!!!!
        if (selenium.isAlertPresent()) {
            selenium.getAlert();
        }
        currentFocusedComponent = null;
        release();
        selenium.open(url);
        waitForCondition();
    }

    /**
     * @see org.testatoo.cartridge.html4.HtmlEvaluator
     */
    @Override
    public String attribute(String id, Attribute attribute) {
        String attributeValue = selenium.getEval("window.tQuery('#" + id + "').attributeValue('" + attribute + "');");
        if (attributeValue.equals("null")) {
            return "";
        }
        return attributeValue;
    }

    /**
     * @see org.testatoo.cartridge.html4.HtmlEvaluator
     */
    @Override
    public Boolean exist(String id, Attribute attribute) {
        return !selenium.getEval("window.tQuery('#" + id + "[" + attribute + "]')").isEmpty();
    }

    /**
     * @see org.testatoo.cartridge.html4.HtmlEvaluator
     */
    @Override
    public String pageSource() {
        return selenium.getHtmlSource();
    }

    /**
     * @see org.testatoo.cartridge.html4.HtmlEvaluator
     */
    @Override
    public List<OptionGroup> optionGroups(Select select) {
        List<OptionGroup> optionGroups = new ArrayList<OptionGroup>();
        try {
            for (Component component : By.jQuery("#" + select.id() + " optgroup").findComponents(this)) {
                optionGroups.add(new OptionGroup(this, component.id()));
            }
        } catch (EvaluatorException e) {
            return optionGroups;
        }
        return optionGroups;
    }

    /**
     * @see org.testatoo.cartridge.html4.HtmlEvaluator
     */
    @Override
    public List<Option> options(Select select) {
        List<Option> options = new ArrayList<Option>();
        try {
            for (Component component : By.jQuery("#" + select.id() + " option").findComponents(this)) {
                options.add(new Option(this, component.id()));
            }
        } catch (EvaluatorException e) {
            return options;
        }
        return options;
    }

    /**
     * @see org.testatoo.cartridge.html4.HtmlEvaluator
     */
    @Override
    public List<Option> selectedOptions(Select select) {
        try {
            String[] selectedIds = elementsId("jquery:#" + select.id() + " option:selected");
            List<Option> options = new ArrayList<Option>();
            for (String id : selectedIds) {
                options.add(new Option(this, id));
            }
            return options;
        }
        catch (EvaluatorException se) {
            return new ArrayList<Option>();
        }
    }

    /**
     * @see org.testatoo.cartridge.html4.HtmlEvaluator
     */
    @Override
    public List<Option> options(OptionGroup optionGroup) {
        List<Option> options = new ArrayList<Option>();
        try {
            for (Component component : By.jQuery("#" + optionGroup.id() + " option").findComponents(this)) {
                options.add(new Option(this, component.id()));
            }
        } catch (EvaluatorException e) {
            return options;
        }
        return options;
    }

    /**
     * @see org.testatoo.cartridge.html4.HtmlEvaluator
     */
    @Override
    public List<Param> params(Object object) {
        List<Param> params = new ArrayList<Param>();
        try {
            for (Component component : By.xpath("//object[@id='" + object.id() + "']//param").findComponents(this)) {
                // TODO maybe bug in jQuery !
//            for (Component component : By.jQuery("#" + object.id() + " param").findComponents(this)) {
                params.add(new Param(this, component.id()));
            }
        } catch (EvaluatorException e) {
            return params;
        }
        return params;
    }

    /**
     * @see org.testatoo.cartridge.html4.HtmlEvaluator
     */
    @Override
    public List<Area> areas(Map map) {
        List<Area> areas = new ArrayList<Area>();
        try {
            for (Component component : By.jQuery("#" + map.id() + " area").findComponents(this)) {
                areas.add(new Area(this, component.id()));
            }
        } catch (EvaluatorException e) {
            return areas;
        }
        return areas;
    }

    /**
     * @see org.testatoo.cartridge.html4.HtmlEvaluator
     */
    @Override
    public List<Col> cols(Colgroup colgroup) {
        List<Col> cols = new ArrayList<Col>();
        try {
            for (Component component : By.jQuery("#" + colgroup.id() + " col").findComponents(this)) {
                cols.add(new Col(this, component.id()));
            }
        } catch (EvaluatorException e) {
            return cols;
        }
        return cols;
    }

    /**
     * @see org.testatoo.cartridge.html4.HtmlEvaluator
     */
    @Override
    public Caption caption(Table table) {
        Component component = By.jQuery("#" + table.id() + " caption").findComponent(this);
        return new Caption(this, component.id());
    }

    /**
     * @see org.testatoo.cartridge.html4.HtmlEvaluator
     */
    @Override
    public String content(Component component) {
        return nodeTextContent(component);
    }

    /**
     * @see org.testatoo.cartridge.html4.HtmlEvaluator
     */
    @Override
    public THead thead(Table table) {
        Component component = By.jQuery("#" + table.id() + " thead").findComponent(this);
        return new THead(this, component.id());
    }

    /**
     * @see org.testatoo.cartridge.html4.HtmlEvaluator
     */
    @Override
    public TBody tbody(Table table) {
        Component component = By.jQuery("#" + table.id() + " tbody").findComponent(this);
        return new TBody(this, component.id());
    }

    /**
     * @see org.testatoo.cartridge.html4.HtmlEvaluator
     */
    @Override
    public TFoot tfoot(Table table) {
        Component component = By.jQuery("#" + table.id() + " tfoot").findComponent(this);
        return new TFoot(this, component.id());
    }

    /**
     * @see org.testatoo.cartridge.html4.HtmlEvaluator
     */
    @Override
    public List<Tr> tr(Component component) {
        List<Tr> tableRows = new ArrayList<Tr>();
        try {
            for (Component _component : By.jQuery("#" + component.id() + " tr").findComponents(this)) {
                tableRows.add(new Tr(this, _component.id()));
            }
        } catch (EvaluatorException e) {
            return tableRows;
        }
        return tableRows;
    }

    /**
     * @see org.testatoo.cartridge.html4.HtmlEvaluator
     */
    @Override
    public List<Td> td(Tr tr) {
        List<Td> td = new ArrayList<Td>();
        try {
            for (Component component : By.jQuery("#" + tr.id() + " td").findComponents(this)) {
                td.add(new Td(this, component.id()));
            }
        } catch (EvaluatorException e) {
            return td;
        }
        return td;
    }

    /**
     * @see org.testatoo.cartridge.html4.HtmlEvaluator
     */
    @Override
    public List<Th> th(Tr tr) {
        List<Th> th = new ArrayList<Th>();
        try {
            for (Component component : By.jQuery("#" + tr.id() + " th").findComponents(this)) {
                th.add(new Th(this, component.id()));
            }
        } catch (EvaluatorException e) {
            return th;
        }
        return th;
    }

    /**
     * @see org.testatoo.cartridge.html4.HtmlEvaluator
     */
    @Override
    public List<Col> cols(Table table) {
        List<Col> cols = new ArrayList<Col>();
        try {
            for (Component component : By.jQuery("#" + table.id() + " col").findComponents(this)) {
                cols.add(new Col(this, component.id()));
            }
        } catch (EvaluatorException e) {
            return cols;
        }
        return cols;
    }

    /**
     * @see org.testatoo.cartridge.html4.HtmlEvaluator
     */
    @Override
    public List<Colgroup> colgroups(Table table) {
        List<Colgroup> colgroups = new ArrayList<Colgroup>();
        try {
            for (Component component : By.jQuery("#" + table.id() + " colgroup").findComponents(this)) {
                colgroups.add(new Colgroup(this, component.id()));
            }
        } catch (EvaluatorException e) {
            return colgroups;
        }
        return colgroups;
    }

    /**
     * @see org.testatoo.cartridge.html4.HtmlEvaluator
     */
    @Override
    public String content(Option option) {
        return nodeTextContent(option);
    }

    /**
     * @see org.testatoo.cartridge.html4.HtmlEvaluator
     */
    @Override
    public void submit(Form form) {
        selenium.getEval("window.tQuery('#" + form.id() + "')[0].submit();");
        waitForCondition();
    }

    /**
     * @see org.testatoo.cartridge.html4.HtmlEvaluator
     */
    @Override
    public void reset(Form form) {
        click(getResetButton(form), Click.left);
        waitForCondition();
    }

    /**
     * @see org.testatoo.cartridge.html4.HtmlEvaluator
     */
    @Override
    public Boolean isReadOnly(Field field) {
        return !selenium.isEditable(field.id());
    }

    /**
     * @see org.testatoo.cartridge.html4.HtmlEvaluator
     */
    @Override
    public String elementId(String expression) {
        String id = null;

        if (expression.startsWith("xpath:")) {
            String xpath = expression.substring(6, expression.length());
            if (!selenium.isElementPresent(xpath)) {
                throw new EvaluatorException("Cannot find component defined by the xpath : " + xpath);
            }
            try {
                id = selenium.getAttribute(xpath + "@id");
            } catch (Exception e) {
                // Ok exists but without identifier so create one
                id = UUID.randomUUID().toString();
                selenium.assignId(xpath, id);
            }
        }

        if (expression.startsWith("jquery:")) {
            String query = expression.substring(7, expression.length());
            if (!Boolean.valueOf(selenium.getEval("window.tQuery('" + query + "').length > 0"))) {
                throw new EvaluatorException("Cannot find component defined by the jquery expression : " + query);
            }

            id = selenium.getEval("window.tQuery('" + query + "').attr('id')");
            if (id.isEmpty()) {
                // Ok exists but without identifier so create one
                id = UUID.randomUUID().toString();
                selenium.getEval("window.tQuery('" + query + "').attr('id', '" + id + "')");
            }
        }
        return id;
    }

    /**
     * @see org.testatoo.cartridge.html4.HtmlEvaluator
     */
    @Override
    public String[] elementsId(String expression) {

        if (expression.startsWith("xpath:")) {
            String xpath = expression.substring(6, expression.length());
            if (!selenium.isElementPresent(xpath)) {
                throw new EvaluatorException("Cannot find component defined by the xpath : " + xpath);
            }

            String[] resultId = extractId(expression);
            for (int i = 0; i < resultId.length; i++) {
                String id = resultId[i];
                if (id.equals("")) {
                    id = UUID.randomUUID().toString();
                    selenium.assignId(xpath + "[" + (i + 1) + "]", id);
                    resultId[i] = id;
                }
            }
            return resultId;
        }

        if (expression.startsWith("jquery:")) {
            String query = expression.substring(7, expression.length());
            if (!Boolean.valueOf(selenium.getEval("window.tQuery('" + query + "').length > 0"))) {
                throw new EvaluatorException("Cannot find component defined by the jquery expression : " + query);
            }

            String[] resultId = extractId(expression);
            for (int i = 0; i < resultId.length; i++) {
                String id = resultId[i];
                if (id.equals("")) {
                    id = UUID.randomUUID().toString();
                    selenium.getEval("window.tQuery(window.tQuery('" + query + "')[" + i + "]).attr('id', '" + id + "')");
                    resultId[i] = id;
                }
            }
            return resultId;
        }

        throw new IllegalArgumentException("The expression format is not supported");
    }

    /**
     * @see org.testatoo.cartridge.html4.HtmlEvaluator
     */
    @Override
    public HtmlElementType htmlElementType(String id) {
        return HtmlElementType.valueOfIgnoreCase(selenium.getEval("window.tQuery('#" + id + "').htmlType()"));
    }

    /**
     * @see org.testatoo.cartridge.html4.HtmlEvaluator
     */
    @Override
    public String nodeTextContent(Component component) {
        return selenium.getText(component.id());
    }

    @Override
    public String toString() {
        final StringBuilder sb = new StringBuilder();
        sb.append("SeleniumHtmlEvaluator");
        sb.append("{location='").append(selenium.getLocation()).append('\'');
        sb.append('}');
        return sb.toString();
    }

    // -------------- Private ----------------------

    private String[] extractId(String expression) {
        if (expression.startsWith("xpath:")) {
            String xpath = expression.substring(6, expression.length());
            return parseCSV(selenium.getEval(
                    "var elements = eval_xpath(\"" + xpath + "\", window.document);" +
                            "var result = [];\n" +
                            "for (var i = 0; i < elements.length; i++) {\n" +
                            "  id = elements[i].id;\n" +
                            "  if (id == '') {id = 'undefined'}; \n" +
                            "  result.push(id);\n" +
                            "}\n" +
                            "result;"));
        }

        if (expression.startsWith("jquery:")) {
            expression = expression.substring(7, expression.length());
            return parseCSV(selenium.getEval(
                    "var elements = window.tQuery(\"" + expression + "\");" +
                            "var result = [];\n" +
                            "for (var i = 0; i < elements.length; i++) {\n" +
                            "  id = elements[i].id;\n" +
                            "  if (id == '') {id = 'undefined'}; \n" +
                            "  result.push(id);\n" +
                            "}\n" +
                            "result;"));
        }
        return null;
    }

    private void setFocus(Component component) {
        if (component instanceof Link || component instanceof Area || component instanceof Button
                || component instanceof Object || component instanceof ListModel || component instanceof Field) {
            selenium.getEval("window.tQuery('#" + component.id() + "').focus()");
            currentFocusedComponent = component;
        }
    }

    private void typeKey(int keyCode) {
        String keyModifier = keyModifier();
        selenium.getEval("if(window.tQuery.browser.webkit) {window.tQuery(window.document).simulate('type', {charCode: " + keyCode + keyModifier + "});}" +
                "else {window.tQuery('body').simulate('type', {keyCode: " + keyCode + keyModifier + "});}");
    }

    private String keyModifier() {
        if (!pressedKeyModifier.isEmpty()) {
            List<String> options = new ArrayList<String>();
            if (pressedKeyModifier.contains(CONTROL)) {
                options.add("ctrlKey : true");
            }
            if (pressedKeyModifier.contains(SHIFT)) {
                options.add("shiftKey : true");
            }
            if (pressedKeyModifier.contains(ALT)) {
                options.add("altKey : true");
            }

            String result = "";
            for (String option : options) {
                result = result + ", " + option;
            }
            return result;
        } else {
            return "";
        }
    }

    private static String[] parseCSV(String input) {
        String[] splitedInput = input.split(",");
        for (int i = 0; i < splitedInput.length; i++) {
            if (splitedInput[i].equalsIgnoreCase("undefined"))
                splitedInput[i] = "";
        }
        return splitedInput;
    }

    private Button getResetButton(Form form) {
        Component reset = By.jQuery("#" + form.id() + " :reset").findComponent(this);
        return new Button(this, reset.id());
    }

    private void waitForCondition() {
        new Wait() {
            public boolean until() {
                return getWaitingCondition().isReach();
            }
        }.wait("One of the waiting conditions has fail", 60000);
    }

    private Select findEmbeddedSelect(ListModel listModel) {
        try {
            if (listModel instanceof Select) {
                return (Select) listModel;
            } else {
                ListBox listBox = (ListBox) listModel;
                java.lang.reflect.Field fields[] = listBox.getClass().getDeclaredFields();
                for (java.lang.reflect.Field field : fields) {
                    field.setAccessible(true);
                    if (field.getName().equals("select")) {
                        return (Select) field.get(listBox);
                    }
                }
            }
        } catch (Exception e) {
            // Nop
        }
        throw new EvaluatorException("Unable to identify the type of ListModel");
    }

}
